%option c++
%option yyclass="GotoLexer"
%option noyywrap
%option nostdinit
%option yylineno
%option case-insensitive
%option full
%option prefix="GOTO_"

%{
    #include <iostream>
    #include <sstream>
    #include "parser.hpp"
    #include "gotolexer.h"
%}
%% // --------------------------------------------------------------------------
"//"[^\n]*\n                            {}
"/*"([^\*]*(\*+)[^*/])*([^\*]*)(\*+)"/" {}
[\n\t" "]*                              {}
";"                                     {return yy::GotoParser::token::SEMICOLON;}
":"                                     {return yy::GotoParser::token::COLON;}
"if"                                    {return yy::GotoParser::token::IF;}
"then"                                  {return yy::GotoParser::token::THEN;}
"goto"                                  {return yy::GotoParser::token::GOTO;}
"stop"                                  {return yy::GotoParser::token::STOP;}
":="                                    {return yy::GotoParser::token::ASSIGN;}
"+"                                     {return yy::GotoParser::token::ADD;}
"-"                                     {return yy::GotoParser::token::SUB;}
"="                                     {return yy::GotoParser::token::EQ;}
[a-zA-Z][a-zA-Z0-9]*                    {
                                          yylval->stringval = new std::string(yytext);
                                          return yy::GotoParser::token::ID;
                                        }
[0-9]+                                  {
                                          yylval->intval = new int;
                                          std::stringstream s;
                                          s << yytext;
                                          s >> *(yylval->intval);
                                          return yy::GotoParser::token::NUM;
                                        }
.|\n                                    {
                                          std::cerr << "\n\nLEXICAL ERROR (Line "
                                            << yylineno << "): Unexpected character \""
                                            << yytext[0] << "\"\n";
                                        }
%% //---------------------------------------------------------------------------

GotoLexer::GotoLexer(std::istream& is)
    : GOTO_FlexLexer(&is)
{
}

int GotoLexer::yylex(yy::GotoParser::semantic_type* lval)
{
    yylval = lval;
    return yylex();
}

int GotoLexer::GetCurrentLineNumber()
{
    return yylineno;
}
